import rpyc
import threading
import time
import Queue
import config
import transactions
import blockChain
from collections import deque

from rpyc.utils.server import ThreadedServer
from keyManagement import loadKeypair, loadPublicKey

LASTINDEXEDBLOCK = ''
T0INDEX = {}
T1INDEX = {}
T0ORDER = deque()
PENDINGTRANSACTIONS = Queue.Queue()
KEY = loadKeypair(config.NODE_KEY_PAIR)
KEY_P = loadPublicKey(config.NODE_PUB_KEY, DER=False)

def monitorTransactions():
    global PENDINGTRANSACTIONS
    global KEY
    global KEY_P
    while(True):
        time.sleep(config.NODE_TRANSACTION_TIMER)
        pendingtransactions = PENDINGTRANSACTIONS
        PENDINGTRANSACTIONS = Queue.Queue()
        candidates = []
        while(True):
            try:
                t = pendingtransactions.get(False)
                candidates.append(t)
            except:
                break
        if len(candidates) > 0:
            #CONSENSUS OVER CANDIDATES NEEDS TO HAPPEN HERE

            #CONSENSUS OVER CANDIDATES NEEDS TO END HERE
            blockChain.appendBlock(KEY_P, blockChain.createBlock(KEY, candidates))
            buildIndexes()

def validateTransaction(transaction):
    try:
        global T0INDEX
        global T1INDEX
        #Check transaction validity
        if not transactions.checkTransaction(transaction): raise

        transactionData = transactions.decodeTransaction(transaction)

        if transactionData[u'type'] == 0:
            #Check if duplicated
            if str(transaction[0]) in T0INDEX: raise

            #Check if command is acceptable (very basic, probably unsafe for production)
            if any(k in '`><$|;\n' for k in transactionData[u'command']): raise
            if transactionData[u'command'].split()[0] not in ['tacker', 'openstack', 'neutron', 'glance', 'nova', 'heat']: raise

        if transactionData[u'type'] == 1:
            #Check if duplicated
            if str(transaction[0]) in T1INDEX: raise

            #Check if 'response_to' field (type=1) points to existing transaction
            if not str(transactionData[u'response_to']) in T0INDEX: raise

        return True
    except:
        return False

def buildIndexes():
    global T0INDEX
    global T1INDEX
    global T0ORDER
    global LASTINDEXEDBLOCK
    last = blockChain.getBlock('last')
    t0order = deque()
    while(True):
        if str(last) == LASTINDEXEDBLOCK: break
        lastBlock = blockChain.decodeBlockData(blockChain.getBlock(last))
        for k in reversed(range(len(lastBlock[u'trans']))):
            transactionData = transactions.decodeTransaction(lastBlock[u'trans'][k])
            # build index t-sig:{'transaction':(b-sig,num),'response':(b-sig,num)} for type 0 transactions
            # build index t-sig:(b-sig,num)} for type 1 transactions
            if transactionData[u'type'] == 0:
                if not str(lastBlock[u'trans'][k][0]) in T0INDEX: T0INDEX[str(lastBlock[u'trans'][k][0])] = {}
                T0INDEX[str(lastBlock[u'trans'][k][0])]['transaction'] = (str(last), k)
                t0order.appendleft(str(lastBlock[u'trans'][k][0]))
            elif transactionData[u'type'] == 1:
                if not str(transactionData[u'response_to']) in T0INDEX: T0INDEX[str(transactionData[u'response_to'])] = {}
                T0INDEX[str(transactionData[u'response_to'])]['response'] = (str(last), k)
                T1INDEX[str(lastBlock[u'trans'][k][0])] = (str(last), k)
        last = lastBlock[u'last']
    LASTINDEXEDBLOCK = blockChain.getBlock('last')
    T0ORDER.extend(t0order)

class BChainService(rpyc.Service):
    def on_connect(self):
        pass

    def on_disconnect(self):
        pass

    def exposed_sendTransaction(self, transaction):
        if not validateTransaction(transaction): return False
        else:
            PENDINGTRANSACTIONS.put(transaction)
            return True

    def exposed_getTransaction(self, sig):
        global T0INDEX
        if sig in T0INDEX:
            if 'transaction' in T0INDEX[sig]:
                blockData = blockChain.decodeBlockData(blockChain.getBlock(T0INDEX[sig]['transaction'][0]))
                return tuple(map(str, blockData[u'trans'][T0INDEX[sig]['transaction'][1]]))
        return None

    def exposed_getTransactionResponse(self, sig):
        global T0INDEX
        if sig in T0INDEX:
            if 'response' in T0INDEX[sig]:
                blockData = blockChain.decodeBlockData(blockChain.getBlock(T0INDEX[sig]['response'][0]))
                return tuple(map(str, blockData[u'trans'][T0INDEX[sig]['response'][1]]))
        return None

    def exposed_getLastTransaction(self):
        global T0ORDER
        sig = T0ORDER[-1]
        return self.exposed_getTransaction(sig)

    def exposed_getTransactionsAfter(self, sig):
        global T0ORDER
        pending = []
        for t in reversed(T0ORDER):
            if t == sig: break
            else: pending.append(self.exposed_getTransaction(t))
        return list(reversed(pending))


def main():
    blockChain.loadChain()                                             #Load chain
    buildIndexes()                                                     #Create memory indexes
    threading.Thread(target=monitorTransactions).start()               #Start transaction monitor
    server = ThreadedServer(BChainService, port=config.NODE_PORT)      #Create listen service
    threading.Thread(target=server.start).start()                      #Start listen service


if __name__ == '__main__':
    main()
